/*
 * $Id: livemanager.c,v 1.2 2002/03/15 17:07:36 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */

/**
 * Speed Barricade live camera manager.
 *
 * @version $Revision: 1.2 $
 */
#include "livemanager.h"
#include "screen.h"
#include "sincos.h"

#define MOTOR_MAX 4
Motor motor[MOTOR_MAX];

#define FRAG_MAX 20
Frag frag[FRAG_MAX];
static int fragIdx = FRAG_MAX;

void initLive() {
  int i, j;
  for ( i=0 ; i<MOTOR_MAX ; i++ ) {
    motor[i].d = NOT_EXIST;
    motor[i].x = motor[i].y = 0;
    for ( j=0 ; j<WALL_MAX ; j++ ) {
      motor[i].wall[j].x1 = NOT_EXIST;
    }
  }
  for ( i=0 ; i<FRAG_MAX ; i++ ) {
    frag[i].cnt = 0;
  }
}

static void addFrag(int x, int y, int z, int mx, int my, int mz, int cnt, int size) {
  Frag *fr;
  fragIdx--; if ( fragIdx < 0 ) fragIdx = FRAG_MAX-1;
  fr = &(frag[fragIdx]);
  fr->x = x; fr->y = y; fr ->z = z;
  fr->mx = mx; fr->my = my; fr ->mz = mz;
  fr->d1 = randN(360); fr->d2 = randN(360);
  fr->md1 = 8+randN(32); fr->md2 = 8+randN(32);
  fr->cnt = cnt; fr->size = size*128;
}

// Set initial motor position.
void setMotor(int mtrIdx, int wlIdx, int x, int y, int d) {
  Motor *mt;
  Wall *wl;
  if ( mtrIdx < 0 || mtrIdx >= MOTOR_MAX || wlIdx < 0 || wlIdx >= WALL_MAX ) return;
  mt = &(motor[mtrIdx]);
  wl = &(mt->wall[wlIdx]);
  mt->d = d;
  mt->wallIdx = wlIdx;
  mt->x = wl->x1 = wl->x2 = x;
  mt->y = wl->y1 = wl->y2 = y;
  wl->downCnt = -1;
}

// Update motor position.
void updateMotor(int mtrIdx, int wlIdx, int x, int y, int d) {
  Wall *wl;
  int wi;
  if ( mtrIdx < 0 || mtrIdx >= MOTOR_MAX || wlIdx < 0 || wlIdx >= WALL_MAX ) return;
  wi = wlIdx-1;
  if ( wi < 0 ) wi = WALL_MAX-2;
  wl = &(motor[mtrIdx].wall[wi]);
  if ( wl->x1 != NOT_EXIST && (wl->x1 == x || wl->y1 == y) ) {
    wl->x1 = x; wl->y1 = y;
  }
  setMotor(mtrIdx, wlIdx, x, y, d);
}

// Wall destroied.
void destroyWall(int mtrIdx, int wlIdx) {
  if ( mtrIdx < 0 || mtrIdx >= MOTOR_MAX || wlIdx < 0 || wlIdx >= WALL_MAX ) return;
  motor[mtrIdx].wall[wlIdx].x1 = NOT_EXIST;
}

// Motor crashed intn a wall.
void crashMotor(int mtrIdx, int x, int y) {
  int i;
  for ( i=0 ; i<8+randN(8) ; i++ ) {
    addFrag(x, 0, y, randNS(512), randN(256), randNS(512), 16+randN(48), 5);
  }
}

// Motor destroied.
void deadMotor(int mtrIdx, int x, int y) {
  int i;
  Motor *mt = &(motor[mtrIdx]);
  mt->d = NOT_EXIST;
  for ( i=0 ; i<WALL_MAX ; i++ ) {
    mt->wall[i].downCnt = 0;
  }
  crashMotor(mtrIdx, x, y);
}

// Reset game status.
void resetGame() {
  initLive();
}


// Handle viewpoint.

#define WALL_HEIGHT 384
#define WALL_DOWN_CNT 64

#define EYE_ROUND 0
#define EYE_FIX 1
#define EYE_CHASE 2
#define EYE_UPON 3

static int eyePtn, eyeCnt = 0, eyeMotorIdx, eyePrm1, eyePrm2, eyePrm3;
static float aimEyeX, aimEyeY, aimEyeZ;

static void moveEyePos() {
  Motor *mt;
  int d;
  eyeCnt--;
  if ( eyeCnt < 0 ) {
    int r;
    eyePtn = randN(4);
    eyeCnt = 120+randN(240);
    if ( randN(3) != 0 ) eyeMotorIdx = 0;
    else eyeMotorIdx = randN(MOTOR_MAX);
    switch ( eyePtn ) {
    case EYE_ROUND:
      eyePrm1 = randN(256);
      eyePrm2 = randNS(2);
      aimEyeY = 2048+randN(3192);
      break;
    case EYE_FIX:
      mt = &(motor[eyeMotorIdx]);
      d = mt->d; if ( d == NOT_EXIST ) d = randN(4);
      aimEyeY = 768 + randN(2048);
      eyePrm3 = 2+randN(10);
      aimEyeX = mt->x; aimEyeZ = mt->y; 
      r = 4096 + randN(16384);
      switch ( d ) {
      case 0:
	aimEyeZ += r;
	break;
      case 1:
	aimEyeX += r;
	break;
      case 2:
	aimEyeZ -= r;
	break;
      case 3:
	aimEyeX -= r;
	break;
      }
      aimEyeX += randNS(2048);
      aimEyeZ += randNS(2048);
      break;
    case EYE_CHASE:
      aimEyeY = 512 + randN(1024);
      eyePrm1 = 8192 + randN(16384);
      eyePrm2 = 64 + randN(128);
      eyePrm3 = randN(6);
      break;
    case EYE_UPON:
      mt = &(motor[eyeMotorIdx]);
      aimEyeX = mt->x + randNS(8192);
      aimEyeZ = mt->y + randNS(8192);
      aimEyeY = 5120 + randN(16384);
      break;
    }
  }
  mt = &(motor[eyeMotorIdx]);
  lookX += (mt->x-lookX)*0.1f; lookZ += (mt->y-lookZ)*0.1f;
  lookY = WALL_HEIGHT/2;
  switch ( eyePtn ) {
  case EYE_ROUND:
    aimEyeX = mt->x + (sctbl[eyePrm1]<<5);
    aimEyeZ = mt->y - (sctbl[eyePrm1+64]<<5);
    eyePrm1 += eyePrm2;
    if ( eyePrm1 >= 256 ) eyePrm1 -= 256;
    else if ( eyePrm1 < 0 ) eyePrm1 += 256;
    break;
  case EYE_FIX:
    aimEyeY += eyePrm3;
    break;
  case EYE_CHASE:
    d = mt->d; if ( d == NOT_EXIST ) d = 0;
    aimEyeX = mt->x; aimEyeZ = mt->y; 
    switch ( d ) {
    case 0:
      aimEyeZ += eyePrm1;
      break;
    case 1:
      aimEyeX += eyePrm1;
      break;
    case 2:
      aimEyeZ -= eyePrm1;
      break;
    case 3:
      aimEyeX -= eyePrm1;
      break;
    }
    aimEyeX += 1024; aimEyeZ += 1024;
    aimEyeY += eyePrm3;
    eyePrm1 -= eyePrm2;
    break;
  case EYE_UPON:
    break;
  }
  eyeX += (aimEyeX-eyeX)*0.05f;  eyeY += (aimEyeY-eyeY)*0.05f; eyeZ += (aimEyeZ-eyeZ)*0.05f; 
}


#define MOTOR_SPEED 192

static void moveMotors() {
  Motor *mt;
  Wall *wl;
  int i, j;
  int mx, my;
  for ( i=0 ; i<MOTOR_MAX ; i++ ) {
    for ( j=0 ; j<WALL_MAX ; j++ ) {
      if ( motor[i].wall[j].x1 == NOT_EXIST ) continue;
      wl = &(motor[i].wall[j]);
      if ( wl->downCnt >= 0 ) {
	wl->downCnt++;
	if ( wl->downCnt >= WALL_DOWN_CNT ) {
	  wl->x1 = NOT_EXIST;
	}
      }
    }
    if ( motor[i].d == NOT_EXIST ) continue;
    mt = &(motor[i]);
    switch ( motor[i].d ) {
    case 0:
      mx = 0;
      my =  MOTOR_SPEED;
      break;
    case 1:
      mx =  MOTOR_SPEED;
      my = 0;
      break;
    case 2:
      mx = 0;
      my = -MOTOR_SPEED;
      break;
    case 3:
      mx = -MOTOR_SPEED;
      my = 0;
      break;
    }
    wl = &(mt->wall[motor[i].wallIdx]);
    if ( wl->x1 == NOT_EXIST ) continue;
    mt->x += mx; mt->y += my;
    if ( mt->x > FIELD_WIDTH*0.7 || mt->x < -FIELD_WIDTH*0.7 || 
	 mt->y > FIELD_WIDTH*0.7 || mt->y < -FIELD_WIDTH*0.7 ) {
      // Motor position is invalid. Perhaps an usb was disconnected.
      usbDisconnected();
      return;
    }
    wl->x1 = mt->x; wl->y1 = mt->y;
    if ( randN(8) == 0 ) {
      addFrag(mt->x, 0, mt->y,
	      (-mx+randNS(64))>>1, randN(64), (-my+randNS(64))>>1, 
	      24+randN(24), 1);
    }
  }
}

static void moveFrags() {
  Frag *fr;
  int i;
  for ( i=0 ; i<FRAG_MAX ; i++ ) {
    if ( frag[i].cnt <= 0 ) continue;
    fr = &(frag[i]);
    fr->x += fr->mx; fr->y += fr->my; fr->z += fr->mz;
    fr->d1 += fr->md1; fr->d2 += fr->md2;
    fr->cnt--;
  }
}

void move() {
  moveMotors();
  moveFrags();
  moveEyePos();
}


static int wallColor[MOTOR_MAX][3] = {
  {240, 120, 120}, {120, 120, 240}, {120, 240, 120}, {240, 240, 120}
};

static void drawWalls() {
  Motor *mt;
  Wall *wl;
  int i, j;
  startDrawWalls();
  for ( i=0 ; i<MOTOR_MAX ; i++ ) {
    mt = &(motor[i]);
    setColor(wallColor[i][0], wallColor[i][1], wallColor[i][2], 220);
    for ( j=0 ; j<WALL_MAX ; j++ ) {
      wl = &(mt->wall[j]);
      if ( wl->x1 == NOT_EXIST ) continue;
      drawBar(wl->x1, wl->y1, wl->x2, wl->y2, 
	      WALL_HEIGHT - wl->downCnt*(WALL_HEIGHT/WALL_DOWN_CNT));
    }
    setColor(wallColor[i][0], wallColor[i][1], wallColor[i][2], 255);
    for ( j=0 ; j<WALL_MAX ; j++ ) {
      wl = &(mt->wall[j]);
      if ( wl->x1 == NOT_EXIST ) continue;
      drawLine(wl->x1, wl->y1, wl->x2, wl->y2, 
	      WALL_HEIGHT - wl->downCnt*(WALL_HEIGHT/WALL_DOWN_CNT));
    }
  }
}

static void drawFrags() {
  Frag *fr;
  int i;
  for ( i=0 ; i<FRAG_MAX ; i++ ) {
    if ( frag[i].cnt <= 0 ) continue;
    fr = &(frag[i]);
    drawFrag(fr->x, fr->y, fr->z, fr->d1, fr->d2, fr->size);
  }
}

static void drawField() {
  setColor(200, 240, 240, 150);
  drawBar(-FIELD_WIDTH/2, -FIELD_WIDTH/2,  FIELD_WIDTH/2, -FIELD_WIDTH/2, WALL_HEIGHT*2);
  drawBar( FIELD_WIDTH/2, -FIELD_WIDTH/2,  FIELD_WIDTH/2,  FIELD_WIDTH/2, WALL_HEIGHT*2);
  drawBar( FIELD_WIDTH/2,  FIELD_WIDTH/2, -FIELD_WIDTH/2,  FIELD_WIDTH/2, WALL_HEIGHT*2);
  drawBar(-FIELD_WIDTH/2,  FIELD_WIDTH/2, -FIELD_WIDTH/2, -FIELD_WIDTH/2, WALL_HEIGHT*2);
  setColor(200, 240, 240, 255);
  drawLine(-FIELD_WIDTH/2, -FIELD_WIDTH/2,  FIELD_WIDTH/2, -FIELD_WIDTH/2, WALL_HEIGHT*2);
  drawLine( FIELD_WIDTH/2, -FIELD_WIDTH/2,  FIELD_WIDTH/2,  FIELD_WIDTH/2, WALL_HEIGHT*2);
  drawLine( FIELD_WIDTH/2,  FIELD_WIDTH/2, -FIELD_WIDTH/2,  FIELD_WIDTH/2, WALL_HEIGHT*2);
  drawLine(-FIELD_WIDTH/2,  FIELD_WIDTH/2, -FIELD_WIDTH/2, -FIELD_WIDTH/2, WALL_HEIGHT*2);
  endDrawWalls();
  drawTile();
}

void draw() {
  drawWalls();
  drawField();
  drawFrags();
  drawEnv();
}
