/*
 * $Id: screen.c,v 1.2 2002/03/16 03:03:03 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */

/**
 * OpenGL screen handler.
 *
 * @version $Revision: 1.2 $
 */
#include <stdio.h>

#include <GL/gl.h>	// Header File For The OpenGL32 Library
#include <GL/glu.h>	// Header File For The GLu32 Library
#include "SDL.h"

#include "sbrg_live.h"
#include "screen.h"

#define FAR_PLANE 128000

// Init OpenGL with texture, smooth line and alpha.
static void initGL(int width, int height) {
  glViewport(0, 0, width, height);
  glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
  glEnable(GL_TEXTURE_2D);

  glEnable(GL_LINE_SMOOTH);
  glLineWidth(2);

  glBlendFunc(GL_SRC_ALPHA, GL_ONE);
  glEnable(GL_BLEND);

  glDisable(GL_CULL_FACE);
  glDisable(GL_LIGHTING);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0f, (GLfloat)width/(GLfloat)height, 0.1f, FAR_PLANE);
  glMatrixMode(GL_MODELVIEW);
}

// Reset viewport when the screen is resized.
void resized(int width, int height) {
  glViewport(0, 0, width, height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0f, (GLfloat)width/(GLfloat)height, 0.1f, FAR_PLANE);
  glMatrixMode(GL_MODELVIEW);
}

// Load bitmaps and convert to textures.
void loadGLTexture(char *fileName, GLuint *texture) {
  SDL_Surface *surface;

  surface = SDL_LoadBMP(fileName);
  if ( !surface ) {
    fprintf(stderr, "Unable to load texture: %s\n", SDL_GetError());
    SDL_Quit();
    exit(1);
  }

  glGenTextures(1, texture);
  glBindTexture(GL_TEXTURE_2D, *texture);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
  gluBuild2DMipmaps(GL_TEXTURE_2D, 3, surface->w, surface->h, GL_RGB, GL_UNSIGNED_BYTE, surface->pixels);
}

#define SCREEN_WIDTH 640
#define SCREEN_HEIGHT 480

#define PT_SIZE 32

#define TILE_BMP    "tile.bmp"
#define PILLAR_BMP  "pillar.bmp"
#define ENV_BMP     "env.bmp"
#define STAR_BMP    "star.bmp"
#define NOINPUT_BMP "noinput.bmp"
#define LIVE_BMP    "live.bmp"
static GLuint tileTexture, pillarTexture, envTexture, starTexture, noinputTexture, liveTexture;

void initSDL(int windowMode) {
  Uint32 videoFlags;
  char *fontFile;

  /* Initialize SDL for video output */
  if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
    exit(1);
  }

  /* Create a OpenGL screen */
  videoFlags = SDL_OPENGL;
  if ( windowMode ) videoFlags |= SDL_RESIZABLE;
  else              videoFlags |= SDL_FULLSCREEN;
  if ( SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, 0, videoFlags ) == NULL ) {
    fprintf(stderr, "Unable to create OpenGL screen: %s\n", SDL_GetError());
    SDL_Quit();
    exit(2);
  }

  initGL(SCREEN_WIDTH, SCREEN_HEIGHT);

  SDL_WM_SetCaption("Speed barricade live camera", NULL);

  loadGLTexture(TILE_BMP, &tileTexture);
  loadGLTexture(PILLAR_BMP, &pillarTexture);
  loadGLTexture(ENV_BMP, &envTexture);
  loadGLTexture(STAR_BMP, &starTexture);
  loadGLTexture(NOINPUT_BMP, &noinputTexture);
  loadGLTexture(LIVE_BMP, &liveTexture);

  SDL_ShowCursor(SDL_DISABLE);
}

void closeSDL() {
  SDL_ShowCursor(SDL_ENABLE);
}

float eyeX = 0, eyeY = 0, eyeZ = 0;
float lookX = 0, lookY = 0, lookZ = 0;

// Set an eye position and draw a text.
void drawGLSceneStart() {
  glClear(GL_COLOR_BUFFER_BIT);

  glPushMatrix();

  if ( isLiveStart() ) {
    glBindTexture(GL_TEXTURE_2D, liveTexture);
  } else {
    glBindTexture(GL_TEXTURE_2D, noinputTexture);
  }
  glColor4ub(255, 255, 255, 255);
  glBegin(GL_TRIANGLE_FAN);
  glTexCoord2f(0.0f, 0.0f); glVertex3f(30, 100, -250);
  glTexCoord2f(1.0f, 0.0f); glVertex3f(130, 100, -250);
  glTexCoord2f(1.0f, 1.0f); glVertex3f(130, 75, -250);
  glTexCoord2f(0.0f, 1.0f); glVertex3f(30, 75, -250);
  glEnd();

  gluLookAt(eyeX, eyeY, -eyeZ, lookX, lookY, -lookZ, 0.0f, 1.0f, 0.0f);
}

void drawGLSceneEnd() {
  glPopMatrix();
  SDL_GL_SwapBuffers();
}

void startDrawWalls() {
  glDisable(GL_TEXTURE_2D);
}

void endDrawWalls() {
  glEnable(GL_TEXTURE_2D);
}

void setColor(int r, int g, int b, int a) {
  glColor4ub(r, g, b, a);
}

void drawBar(int x1, int y1, int x2, int y2, int height) {
  glBegin(GL_TRIANGLE_FAN);
  glVertex3f(x1, 0,  -y1);
  glVertex3f(x1, height, -y1);
  glVertex3f(x2, height, -y2);
  glVertex3f(x2, 0, -y2);
  glEnd();
}

void drawLine(int x1, int y1, int x2, int y2, int height) {
  glBegin(GL_LINES);
  glVertex3f(x1, height, -y1);
  glVertex3f(x2, height, -y2);
  glEnd();
}

#define PILLAR_WIDTH 784
#define PILLAR_HEIGHT 25600

void drawTile() {
  glBindTexture(GL_TEXTURE_2D, tileTexture);
  glColor4ub(200, 200, 200, 255);
  glBegin(GL_TRIANGLE_FAN);
  glTexCoord2f(0.0f, 0.0f); glVertex3f(-FIELD_WIDTH/2, 0, -FIELD_WIDTH/2);
  glTexCoord2f(7.0f, 0.0f); glVertex3f( FIELD_WIDTH/2, 0, -FIELD_WIDTH/2);
  glTexCoord2f(7.0f, 7.0f); glVertex3f( FIELD_WIDTH/2, 0,  FIELD_WIDTH/2);
  glTexCoord2f(0.0f, 7.0f); glVertex3f(-FIELD_WIDTH/2, 0,  FIELD_WIDTH/2);
  glEnd();
  glBindTexture(GL_TEXTURE_2D, pillarTexture);
  glColor4ub(255, 255, 255, 128);
  glBegin(GL_TRIANGLE_FAN);
  glTexCoord2f(0.0f, 0.05f); glVertex3f(-PILLAR_WIDTH/2, PILLAR_HEIGHT, 0);
  glTexCoord2f(1.0f, 0.05f); glVertex3f( PILLAR_WIDTH/2, PILLAR_HEIGHT, 0);
  glTexCoord2f(1.0f, 1.0f); glVertex3f( PILLAR_WIDTH/2, 0, 0);
  glTexCoord2f(0.0f, 1.0f); glVertex3f(-PILLAR_WIDTH/2, 0, 0);
  glEnd();
  glBegin(GL_TRIANGLE_FAN);
  glTexCoord2f(0.0f, 0.05f); glVertex3f(0, PILLAR_HEIGHT, -PILLAR_WIDTH/2);
  glTexCoord2f(1.0f, 0.05f); glVertex3f(0, PILLAR_HEIGHT,  PILLAR_WIDTH/2);
  glTexCoord2f(1.0f, 1.0f); glVertex3f(0, 0,  PILLAR_WIDTH/2);
  glTexCoord2f(0.0f, 1.0f); glVertex3f(0, 0, -PILLAR_WIDTH/2);
  glEnd();
}

void drawFrag(int x, int y, int z, int d1, int d2, int frSize) {
  int size;
  glBindTexture(GL_TEXTURE_2D, starTexture);
  glPushMatrix();
  glTranslatef(x, y, -z);
  glColor4ub(100+randN(55), 100+randN(55), 100+randN(55), 255);
  glRotatef(d1, 0.0f, 1.0f, 0.0f);
  size = (frSize*(256+randNS(64)))>>8;
  glBegin(GL_TRIANGLE_FAN);
  glTexCoord2f(0.0f, 0.0f); glVertex3f(-size, -size,  0);
  glTexCoord2f(1.0f, 0.0f); glVertex3f( size, -size,  0);
  glTexCoord2f(1.0f, 1.0f); glVertex3f( size,  size,  0);
  glTexCoord2f(0.0f, 1.0f); glVertex3f(-size,  size,  0);
  glColor4ub(100+randN(55), 100+randN(55), 100+randN(55), 255);
  glRotatef(d2, 1.0f, 0.0f, 0.0f);
  size = (frSize*(256+randNS(64)))>>8;
  glBegin(GL_TRIANGLE_FAN);
  glTexCoord2f(0.0f, 0.0f); glVertex3f(-size, -size,  0);
  glTexCoord2f(1.0f, 0.0f); glVertex3f( size, -size,  0);
  glTexCoord2f(1.0f, 1.0f); glVertex3f( size,  size,  0);
  glTexCoord2f(0.0f, 1.0f); glVertex3f(-size,  size,  0);
  glEnd();
  glPopMatrix();
}

#define ENV_PLANE 64000

void drawEnv() {
  glBindTexture(GL_TEXTURE_2D, envTexture);
  glColor4ub(255, 255, 255, 255);
  glPushMatrix();
  glTranslatef(eyeX, eyeY, -eyeZ);
  glBegin(GL_QUADS);
  glTexCoord2f(0.0f, 0.0f); 
  glVertex3f(-ENV_PLANE, -ENV_PLANE, -ENV_PLANE);
  glTexCoord2f(1.0f, 0.0f);
  glVertex3f( ENV_PLANE, -ENV_PLANE, -ENV_PLANE);
  glTexCoord2f(1.0f, 1.0f);
  glVertex3f( ENV_PLANE,  ENV_PLANE, -ENV_PLANE);
  glTexCoord2f(0.0f, 1.0f);
  glVertex3f(-ENV_PLANE,  ENV_PLANE, -ENV_PLANE);

  glTexCoord2f(0.0f, 0.0f); 
  glVertex3f(-ENV_PLANE, -ENV_PLANE,  ENV_PLANE);
  glTexCoord2f(0.0f, 1.0f);
  glVertex3f(-ENV_PLANE,  ENV_PLANE,  ENV_PLANE);
  glTexCoord2f(1.0f, 1.0f);
  glVertex3f( ENV_PLANE,  ENV_PLANE,  ENV_PLANE);
  glTexCoord2f(1.0f, 0.0f);
  glVertex3f( ENV_PLANE, -ENV_PLANE,  ENV_PLANE);

  glTexCoord2f(0.0f, 0.0f); 
  glVertex3f(-ENV_PLANE, -ENV_PLANE, -ENV_PLANE);
  glTexCoord2f(1.0f, 0.0f);
  glVertex3f( ENV_PLANE, -ENV_PLANE, -ENV_PLANE);
  glTexCoord2f(1.0f, 1.0f);
  glVertex3f( ENV_PLANE, -ENV_PLANE,  ENV_PLANE);
  glTexCoord2f(0.0f, 1.0f);
  glVertex3f(-ENV_PLANE, -ENV_PLANE,  ENV_PLANE);

  glTexCoord2f(0.0f, 0.0f); 
  glVertex3f(-ENV_PLANE,  ENV_PLANE, -ENV_PLANE);
  glTexCoord2f(0.0f, 1.0f);
  glVertex3f(-ENV_PLANE,  ENV_PLANE,  ENV_PLANE);
  glTexCoord2f(1.0f, 1.0f);
  glVertex3f( ENV_PLANE,  ENV_PLANE,  ENV_PLANE);
  glTexCoord2f(1.0f, 0.0f);
  glVertex3f( ENV_PLANE,  ENV_PLANE, -ENV_PLANE);

  glTexCoord2f(0.0f, 0.0f); 
  glVertex3f(-ENV_PLANE, -ENV_PLANE, -ENV_PLANE);
  glTexCoord2f(1.0f, 0.0f);
  glVertex3f(-ENV_PLANE,  ENV_PLANE, -ENV_PLANE);
  glTexCoord2f(1.0f, 1.0f);
  glVertex3f(-ENV_PLANE,  ENV_PLANE,  ENV_PLANE);
  glTexCoord2f(0.0f, 1.0f);
  glVertex3f(-ENV_PLANE, -ENV_PLANE,  ENV_PLANE);

  glTexCoord2f(0.0f, 0.0f); 
  glVertex3f( ENV_PLANE, -ENV_PLANE, -ENV_PLANE);
  glTexCoord2f(0.0f, 1.0f);
  glVertex3f( ENV_PLANE, -ENV_PLANE,  ENV_PLANE);
  glTexCoord2f(1.0f, 1.0f);
  glVertex3f( ENV_PLANE,  ENV_PLANE,  ENV_PLANE);
  glTexCoord2f(1.0f, 0.0f);
  glVertex3f( ENV_PLANE,  ENV_PLANE, -ENV_PLANE);

  glEnd();
  glPopMatrix();
}
